@@ -1,6 +1,6 @@
 # Author: Babak Naimi, naimi.b@gmail.com
-# Date (last update):  Nov. 2020
-# Version 3.1
+# Date (last update):  Feb 2024
+# Version 3.3
 # Licence GPL v3
 #---------------------
 
@@ -50,7 +50,7 @@ setMethod("plot", signature(x='sdmEvaluate'),
 
 setMethod("plot", signature(x='sdmdata'),
           function(x,y,sp=NULL,test=FALSE,col=c('blue','red'),xlab,ylab,main,xlim,ylim,pch,...) {
-            xy <- coordinates(x)
+            xy <- coords(x)
             
             if (missing(y)) {
               if (!is.null(xy)) y <- 'map'
@@ -587,6 +587,48 @@ setMethod("plot", signature(x='.nicheRaster'),
             }
           }
 )
+#----
+setMethod("plot", signature(x='.nicheSpatRaster'),
+          function(x,y=NULL,gg=TRUE,rnd=1,xlab,ylab,col,cex.axis,cex.lab,main,...) {
+            if (missing(gg)) gg <- .require('ggplot2')
+            else if (gg && !.require('ggplot2')) gg <- FALSE
+            
+            if (missing(xlab)) xlab <- x@names[1]
+            if (missing(ylab)) ylab <- x@names[2]
+            
+            if (missing(col)) col <- c('darkred','red','yellow','green','darkgreen','darkblue')
+            
+            if (missing(cex.axis)) cex.axis <- 0.8
+            if (missing(cex.lab)) cex.lab <- 1
+            
+            if (missing(rnd)) rnd <- 1
+            
+            if (missing(main)) main <- paste0('Ecological Niche described by: ',x@names[1],' - ',x@names[2])
+            
+            .lab1 <- round(as.vector(seq(x@scaleParams[1,1], x@scaleParams[2,1], length.out = 6)),rnd)
+            .lab2 <- round(as.vector(seq(x@scaleParams[1,2], x@scaleParams[2,2], length.out = 6)),rnd)
+            #--------
+            if (gg) {
+              drc <- as.data.frame(x@nicheRaster,xy=TRUE,na.rm=FALSE)
+              p1 <- "ggplot(drc,aes(x=x,y=y,fill=niche)) +geom_raster() + coord_quickmap() + 
+                    scale_y_continuous(breaks=seq(0, 1, length.out = 6),name = ylab,labels=.lab2) + scale_x_continuous(breaks=seq(0, 1, length.out = 6),name = xlab,labels=.lab1) + ggtitle(main) +
+                    scale_fill_gradientn(colours=col,na.value='white') + theme_bw() +
+                    theme(axis.text=element_text(size=rel(cex.axis)),axis.title=element_text(size=rel(cex.lab)),plot.title = element_text(hjust = 0.5))"
+              p1 <- .eval(p1,env=environment())
+              return(p1)
+            } else {
+              if (length(col) < 10) col <- colorRampPalette(col)(100)
+              
+              plot(x@nicheRaster,col=col,xaxt='n',yaxt='n',xlab=xlab,ylab=ylab,main=main,cex.axis=cex.axis,cex.lab=cex.lab,...)
+              
+              axis(1, at=seq(0, 1, length.out = 6), labels = FALSE)
+              text(seq(0, 1, length.out = 6),par("usr")[3] - 0.05, labels = .lab1, srt = 0, pos = 1, xpd = TRUE,cex=cex.axis)
+              
+              axis(2, at=seq(0, 1, length.out = 6), labels = FALSE)
+              text(par("usr")[1]-0.1,seq(0, 1, length.out = 6)+0.05, labels = .lab2, srt = 0, pos = 1, xpd = TRUE,cex=cex.axis)
+            }
+          }
+)
 
 #--------
 if (!isGeneric("boxplot")) {
